/*
 * debug functions
 */

#include <stdarg.h>
#include <linux/fs.h>
#include <linux/bfs_fs.h>

#ifdef BFS_DEBUG


void bfs_dump_inode (bfs_inode * inode)
{
	printk ("bfs_inode infomation\n");

	printk ("  magic1 %08x\n", inode->magic1);
	printk ("  inode_num %lu, %u %u\n", inode->inode_num.allocation_group,
		inode->inode_num.start, inode->inode_num.len); 
	printk ("  uid %lu\n", inode->uid);
	printk ("  gid %lu\n", inode->gid);
	printk ("  mode %08x\n", inode->mode);
	printk ("  flags %08x\n", inode->flags);
	printk ("  create_time %u\n", inode->create_time >> 8);
	printk ("  last_modified_time %u\n", inode->last_modified_time >> 8);
	printk ("  parent %lu, %u, %u\n", inode->parent.allocation_group,
		inode->parent.start, inode->parent.len);
	printk ("  attributes %lu, %u, %u\n", inode->attributes.allocation_group,
		inode->attributes.start, inode->attributes.len);
	printk ("  type %08x\n", inode->type);
	printk ("  inode_size %lu\n", inode->inode_size);
}



void bfs_dump_super_block (bfs_super_block * bs)
{
	/*
	 * Display super block structure for debug.
	 */
	printk ("bfs_super_block information\n");

	printk ("  name %s\n", bs->name);
	printk ("  magic1 %08x\n", bs->magic1);
	printk ("  fs_byte_order %08x\n", bs->fs_byte_order);

	printk ("  block_size %u\n", bs->block_size);
	printk ("  block_shift %u\n", bs->block_shift);

	printk ("  num_blocks %lu\n", bs->num_blocks);
	printk ("  used_blocks %u\n", bs->used_blocks);

	printk ("  magic2 %08x\n", bs->magic2);
	printk ("  blocks_per_ag %u\n", bs->blocks_per_ag);
	printk ("  ag_shift %u\n", bs->ag_shift);
	printk ("  num_ags %u\n", bs->num_ags);

	printk ("  flags %08x\n", bs->flags);

	printk ("  log_blocks %lu, %u, %u\n",
		bs->log_blocks.allocation_group,
		bs->log_blocks.start, bs->log_blocks.len);
	printk ("  log_start %u\n", bs->log_start);
	printk ("  log_end %u\n", bs->log_end);

	printk ("  magic3 %08x\n", bs->magic3);
	printk ("  root_dir %lu, %u, %u\n",
		bs->root_dir.allocation_group, bs->root_dir.start,
		bs->root_dir.len);
	printk ("  indices %lu, %u, %u\n",
		bs->indices.allocation_group, bs->indices.start,
		bs->indices.len);
}



void bfs_dump_small_data (bfs_small_data * sd)
{
}



void bfs_dump_inode_addr (bfs_inode_addr iaddr)
{
	printk ("  allocation_group %lu\n", iaddr.allocation_group);
	printk ("  start %u\n", iaddr.start);
	printk ("  len %u\n", iaddr.len);
}



void bfs_dump_index_entry (bfs_index_entry * entry)
{
	printk ("index entry structure\n");
	printk ("  magic %08lx\n", entry->magic);
	printk ("  node_size %08x\n", entry->node_size);
	printk ("  max_number_of_levels %08x\n",
		entry->max_number_of_levels);
	printk ("  data_type %08x\n", entry->data_type);
	printk ("  root_node_pointer %Lu\n", entry->root_node_pointer);
	printk ("  free_node_pointer %Lu\n", entry->free_node_pointer);
	printk ("  maximum size %Lu\n", entry->maximum_size);
}


void bfs_dump_index_node (bfs_index_node * node)
{
	printk ("inode of index node structure\n");
	printk ("  left %Lu\n", node->left);
	printk ("  right %Lu\n", node->right);
	printk ("  overflow %Lu\n", node->overflow);
	printk ("  all_key_count %u\n", node->all_key_count);
	printk ("  all_key_length %u\n", node->all_key_length);
}



#endif
