/*
 *  linux/fs/bfs/symlink.c
 *
 * Copyright (C) 1999  Makoto Kato (m_kato@ga2.so-net.ne.jp)
 *
 *
 *  BFS symlink handling code
 */

#include <asm/uaccess.h>

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/bfs_fs.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/stat.h>

static int bfs_readlink (struct dentry *, char *, int);
static struct dentry * bfs_follow_link(struct dentry *, struct dentry *,
	unsigned int);

/*
 * symlinks can't do much...
 */
struct inode_operations bfs_symlink_inode_operations = {
        NULL,                   /* no file-operations */
        NULL,                   /* create */
        NULL,                   /* lookup */
        NULL,                   /* link */
        NULL,                   /* unlink */
        NULL,                   /* symlink */
        NULL,                   /* mkdir */
        NULL,                   /* rmdir */
        NULL,                   /* mknod */
	NULL,			/* rename */
	bfs_readlink,		/* readlink */
        bfs_follow_link,       /* follow_link */
        NULL,                   /* readpage */
        NULL,                   /* writepage */
        NULL,                   /* bmap */
        NULL,                   /* truncate */
        NULL,                   /* permission */
        NULL                    /* smap */
};


/*
 * The inode of symbolic link is different to data stream.
 * The data stream become link name.
 */

static struct dentry * bfs_follow_link(struct dentry * dentry,
	struct dentry * base, unsigned int follow)
{
        struct inode * inode = dentry->d_inode;
        char *         link = inode->u.bfs_i.i_data.symlink;

        UPDATE_ATIME(inode);

        base = lookup_dentry(link, base, follow);

        return base;
}


static int bfs_readlink(struct dentry * dentry, char * buffer, int buflen)
{
        struct inode * inode = dentry->d_inode;
        char *         link = inode->u.bfs_i.i_data.symlink;
        int            i = 0;

        if (buflen > BFS_SYMLINK_LEN)
                buflen = BFS_SYMLINK_LEN;

        while (i < buflen && link[i])
                i++;

        if (copy_to_user(buffer, link, i))
                i = -EFAULT;

        UPDATE_ATIME(inode);

        return i;
}

